@@ -20,6 +20,13 @@ os_win_opts = [
                default='hbaapi.dll',
                help='Fibre Channel hbaapi library path. If no custom hbaapi '
                     'library is requested, the default one will be used.'),
+    cfg.BoolOpt('cache_temporary_wmi_objects',
+                default=True,
+                help='Caches temporary WMI objects in order to increase '
+                     'performance. This only affects networkutils, where '
+                     'almost all operations require a reference to a '
+                     'switch port. The cached objects are no longer valid '
+                     'if the VM they are associated with is destroyed.'),
 ]
 
 CONF = cfg.CONF
