@@ -331,6 +331,31 @@ class NetworkUtilsTestCase(test_base.OsWinBaseTestCase):
         self.assertEqual(expected_cache, self.netutils._switches)
         self.assertEqual(self._FAKE_VSWITCH, vswitch)
 
+    @mock.patch.object(networkutils.NetworkUtils,
+                       '_get_vswitch')
+    def test_get_vswitch_extensions(self, mock_get_vswitch):
+        mock_vswitch = mock_get_vswitch.return_value
+        mock_ext = mock.Mock()
+        ext_cls = self.netutils._conn.Msvm_EthernetSwitchExtension
+        ext_cls.return_value = [mock_ext] * 2
+
+        extensions = self.netutils.get_vswitch_extensions(
+            mock.sentinel.vswitch_name)
+        exp_extensions = [
+            {'name': mock_ext.ElementName,
+             'version': mock_ext.Version,
+             'vendor': mock_ext.Vendor,
+             'description': mock_ext.Description,
+             'enabled_state': mock_ext.EnabledState,
+             'extension_type': mock_ext.ExtensionType}] * 2
+
+        self.assertEqual(exp_extensions, extensions)
+
+        mock_get_vswitch.assert_called_once_with(
+            mock.sentinel.vswitch_name)
+        ext_cls.assert_called_once_with(
+            SystemName=mock_vswitch.Name)
+
     def test_get_vswitch_cache(self):
         self.netutils._switches = {
             self._FAKE_VSWITCH_NAME: mock.sentinel.vswitch}
@@ -573,16 +598,14 @@ class NetworkUtilsTestCase(test_base.OsWinBaseTestCase):
             self.netutils._set_switch_port_security_settings(
                 mock.sentinel.switch_port_name,
                 VirtualSubnetId=mock.sentinel.vsid)
+            mock_remove_feature = self.netutils._jobutils.remove_virt_feature
+            mock_remove_feature.assert_called_once_with(mock_sec_settings)
 
         self.assertEqual(mock.sentinel.vsid,
                          mock_sec_settings.VirtualSubnetId)
-        if missing_sec:
-            mock_add_feature = self.netutils._jobutils.add_virt_feature
-            mock_add_feature.assert_called_once_with(mock_sec_settings,
-                                                     mock_port_alloc)
-        else:
-            mock_modify_feature = self.netutils._jobutils.modify_virt_feature
-            mock_modify_feature.assert_called_once_with(mock_sec_settings)
+        mock_add_feature = self.netutils._jobutils.add_virt_feature
+        mock_add_feature.assert_called_once_with(mock_sec_settings,
+                                                 mock_port_alloc)
 
     def test_set_switch_port_security_settings(self):
         self._check_set_switch_port_security_settings()
