@@ -175,6 +175,22 @@ class NetworkUtils(baseutils.BaseUtilsVirt):
         vswitch = self._get_vswitch(vswitch_name)
         return vswitch.Name
 
+    def get_vswitch_extensions(self, vswitch_name):
+        vswitch = self._get_vswitch(vswitch_name)
+
+        extensions = self._conn.Msvm_EthernetSwitchExtension(
+            SystemName=vswitch.Name)
+        dict_ext_list = [
+            {'name': ext.ElementName,
+             'version': ext.Version,
+             'vendor': ext.Vendor,
+             'description': ext.Description,
+             'enabled_state': ext.EnabledState,
+             'extension_type': ext.ExtensionType}
+            for ext in extensions]
+
+        return dict_ext_list
+
     def get_vswitch_external_network_name(self, vswitch_name):
         ext_port = self._get_vswitch_external_port(vswitch_name)
         if ext_port:
@@ -518,12 +534,14 @@ class NetworkUtils(baseutils.BaseUtilsVirt):
         sec_settings = self._get_security_setting_data_from_port_alloc(
             port_alloc)
 
-        exists = sec_settings is not None
-
-        if exists:
+        if sec_settings:
             if all(getattr(sec_settings, k) == v for k, v in kwargs.items()):
                 # All desired properties already properly set. Nothing to do.
                 return
+
+            # Removing the feature because it cannot be modified
+            # due to a wmi exception.
+            self._jobutils.remove_virt_feature(sec_settings)
         else:
             sec_settings = self._create_default_setting_data(
                 self._PORT_SECURITY_SET_DATA)
@@ -531,10 +549,7 @@ class NetworkUtils(baseutils.BaseUtilsVirt):
         for k, v in kwargs.items():
             setattr(sec_settings, k, v)
 
-        if exists:
-            self._jobutils.modify_virt_feature(sec_settings)
-        else:
-            self._jobutils.add_virt_feature(sec_settings, port_alloc)
+        self._jobutils.add_virt_feature(sec_settings, port_alloc)
 
         # TODO(claudiub): This will help solve the missing VSID issue, but it
         # comes with a performance cost. The root cause of the problem must
